#version 330

in vec3 worldNormal;
in vec2 fUVs;
in vec3 vertexPosition; //light calculation with original values without transformation
in vec4 vertexPosShadowMap;

out vec4 fragColor;

uniform sampler2D colorTexture;
uniform sampler2D paintLayer;
uniform sampler2DShadow shadowMap;
uniform vec3 ambientLight;
uniform vec3 lightPosition; 
uniform vec3 cameraPosition;
uniform vec3 spotPosition;
uniform vec3 spotDirection;
uniform vec3 k_ambient;
uniform vec3 k_diffuse;
uniform vec3 k_specular;

float PCF(sampler2DShadow shadowMap, float offset, vec4 uv);

void main() {

	vec3 V = normalize(cameraPosition - vertexPosition);	//Eye-Vector
	vec3 L = normalize(lightPosition - vertexPosition); //Light-Vector
	vec3 N = normalize(worldNormal);

	//***Blinn-Phong*** Ambient
	vec3 I_ambient = ambientLight * k_ambient;

	//***Blinn-Phong*** Diffuse
	float cos_phi = vec3(max(dot(L,N),0));
//	cos_phi = pow(cos_phi, 1);
	float dist = distance(vertexPosition, lightPosition);
	float Ldist=50/(10+2*dist+pow(dist,2));
	vec3 I_diffuse = vec3(cos_phi) * k_diffuse * Ldist;

	//***Blinn-Phong*** Specular:
	vec3 H = (V+L) / sqrt(dot((V+L),(V+L)));	//Half-Vector
	float shininess = 500;
	float cosTheta = pow(dot(N, H), shininess);	//Blinn substitute
	vec3 I_specular = vec3(clamp(cosTheta,0,1)) * k_specular;

	vec4 textureColor = texture(colorTexture, fUVs).rgba;
	vec4 texturePaintLayer = texture(paintLayer, fUVs).rgba;

	//Spotlight:
	vec3 vDir = vertexPosition - spotPosition;
	vDir = normalize(vDir);
	float fCosine = dot(normalize(spotDirection), vDir);  
	//angle (20) * pi / 180
	float fConeCosine = cos(0.349066);
    
	vec3 spot=vec3(0);
	if(fCosine > fConeCosine)
	{
		float fDistance = distance(vertexPosition, spotPosition);
		float fDif = 1.0-fConeCosine;
		float fFactor = clamp((fCosine-fConeCosine)/fDif, 0.0, 1.0);
		float lightAngle = max(dot(spotPosition-vertexPosition,N),0);
		spot = lightAngle * vec3(fFactor/(fDistance*0.8));
	}

	//Shadow Map
	float visibility = PCF(shadowMap, 0.005, vertexPosShadowMap);

	//Paint Heightmap
	vec3 paintedTexture = textureColor.rgb * (1-texturePaintLayer.a) + texturePaintLayer.rgb * texturePaintLayer.a;

	//Final Output
	fragColor = vec4( paintedTexture * I_ambient + visibility * (paintedTexture * I_diffuse + I_specular) + spot * paintedTexture, textureColor.a);
}

float PCF(sampler2DShadow shadowMap, float offset, vec4 uv)
{
	float bias = 0.0005;
	float result = 0;

	for(int x=-1; x<=1; x++)
	{
		for(int y=-1; y<=1; y++)
		{
			//result += texture( shadowMap, vec3(uv.x + (offset*x), uv.y + (offset*y), (uv.z-bias)/uv.w) );
			//result += textureProj( shadowMap, vec4(uv.x + (offset*x), uv.y + (offset*y), uv.z-bias, uv.w ));
			result += textureProj( shadowMap, vec4(uv.x + (offset*x), uv.y + (offset*y), uv.z-bias, uv.w ));
			//result += texture2DProj( shadowMap, vec3(uv.x + (offset*x), uv.y + (offset*y), (uv.z-bias)/uv.w) );
		}
	}
	return (result / 9);
}